/***************************************************************************
                           soundwidgetimpl.cpp
                           -------------------
    begin                : Mon Jun 3 2002
    copyright            : (C) 2003 by Dirk Ziegelmeier
    email                : dziegel@gmx.de
    rcs data             : $Id: telex.cpp,v 1.12 2003/08/22 07:53:45 malte Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>
#include <klocale.h>
#include "soundwidgetimpl.h"
#include "cfgdata.h"
#include "qvmixerplugin.h"
#include "qtvision.h"
#include "audiomanager.h"

#include <qcombobox.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qtextedit.h>
#include <kdialog.h>
#include <kconfig.h>
#include <qlayout.h>
#include <qsplitter.h>
#include <kpushbutton.h>
#include <qspinbox.h>
#include <qcheckbox.h>


SoundWidgetImpl::SoundWidgetImpl(QWidget* parent, QtVision* qtv)
                :SoundWidget(parent), _qtv(qtv)
{
}

SoundWidgetImpl::~SoundWidgetImpl()
{
}

void SoundWidgetImpl::setup(ConfigData* cfg, KConfig*)
{
    _mixer->clear();
    for (PluginDesc *p = _qtv->pluginFactory()->mixerPlugins().first();
	 p;
	 p = _qtv->pluginFactory()->mixerPlugins().next())
    {
	_mixer->insertItem(p->name);
    }
    for (int i=0; i<_mixer->count(); i++) {
	if (_qtv->audioManager()->plugin()->pluginDescription() == getPluginDesc(_mixer->text(i))) {
	    _mixer->setCurrentItem(i);
	    break;
	}
    }
    mixerSelectionChanged(_mixer->currentText());

    connect(_mixer, SIGNAL(highlighted(const QString&)),
	    this, SLOT(mixerSelectionChanged(const QString&)));
    connect(_configure, SIGNAL(clicked()),
	    this, SLOT(configurePlugin()));

    _volumeIncrement->setValue(cfg->volumeIncrement);
    _delayOnChange->setValue(cfg->volumeRestoreDelay);
    _autoMute->setChecked(cfg->autoMute);
}

void SoundWidgetImpl::mixerSelectionChanged(const QString& mixer)
{
    PluginDesc *p = getPluginDesc(mixer);
    _author->setText(p->author);
    _description->setText(p->comment);
    _configure->setEnabled(p->configurable);
}

PluginDesc* SoundWidgetImpl::getPluginDesc(const QString& mixer)
{
    for (PluginDesc *p = _qtv->pluginFactory()->mixerPlugins().first();
	 p;
	 p = _qtv->pluginFactory()->mixerPlugins().next())
    {
	if (p->name == mixer) return p;
    }
    return 0L;
}

void SoundWidgetImpl::configurePlugin()
{
    KDialog*       kdb      = new KDialog(this, "Mixer plugin config panel");
    QGridLayout*   grid     = new QGridLayout(kdb, 10, 10);
    KPushButton*   ok_b     = new KPushButton(i18n("&Ok"), kdb);
    KPushButton*   cancel_b = new KPushButton(i18n("&Cancel"), kdb);
    QSplitter*     sp       = new QSplitter(QSplitter::Horizontal, kdb);

    PluginDesc*    pd       = getPluginDesc(_mixer->currentText());

    bool previousState      = pd->enabled;
    pd->enabled             = true;
    QVMixerPlugin* p        = _qtv->pluginFactory()->getMixerPlugin(pd);
    pd->enabled             = previousState;
    QWidget*       mw       = p->configWidget(kdb, "Mixer configuration widget");

    grid->addMultiCellWidget(sp, 8, 8, 0, 9);
    grid->addMultiCellWidget(ok_b, 9, 9, 7, 7);
    grid->addMultiCellWidget(cancel_b, 9, 9, 9, 9);
    grid->addMultiCellWidget(mw, 0, 7, 0, 9);
    kdb->setCaption(i18n("Configuration"));
    QSize ms(300,200);
    QSize nh = kdb->sizeHint().expandedTo(ms);
    grid->setResizeMode(QLayout::FreeResize);
    kdb->setMinimumSize(nh);
    kdb->resize(nh);

    connect(ok_b, SIGNAL(clicked()), 
	    kdb, SLOT(accept()));
    connect(cancel_b, SIGNAL(clicked()),
	    kdb, SLOT(reject()));
    
    if (kdb->exec() == QDialog::Accepted) {
	p->saveConfig();
    }
    p->destroy();
}

void SoundWidgetImpl::apply(ConfigData* cfg, KConfig* h)
{
    cfg->autoMute           = _autoMute->isChecked();
    cfg->volumeRestoreDelay = _delayOnChange->value();
    cfg->volumeIncrement    = _volumeIncrement->value();

    h->setGroup("Mixer Plugins");
    for (PluginDesc* p = _qtv->pluginFactory()->mixerPlugins().first();
	 p;
	 p = _qtv->pluginFactory()->mixerPlugins().next())
    {
	if (p->name == _mixer->currentText()) {
	    h->writeEntry(p->name + "-" + p->author, true);
	    p->enabled = true;
	} else {
	    h->writeEntry(p->name + "-" + p->author, false);
	    p->enabled = false;
	}
    }

    _qtv->audioManager()->rescanPlugins();
}

#include "soundwidgetimpl.moc"
