/***************************************************************************
                           videowidgetimpl.cpp
                           -------------------
    begin                : Mon Jun 3 2002
    copyright            : (C) 2003 by Dirk Ziegelmeier
    email                : dziegel@gmx.de
    rcs data             : $Id: $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>
#include <klocale.h>
#include "videowidgetimpl.h"
#include <klistview.h>
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qtextedit.h>
#include <kdialog.h>
#include <kconfig.h>
#include <qlayout.h>
#include <qsplitter.h>
#include <kpushbutton.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include "qtvision.h"
#include "qvsrcplugin.h"


class VideoPluginListItem : public QCheckListItem
{
public:
    VideoPluginListItem(KListView*, PluginDesc*);
    virtual ~VideoPluginListItem();
    
    PluginDesc* pd;
};

VideoPluginListItem::VideoPluginListItem(KListView *parent, PluginDesc *d)
    : QCheckListItem(parent, d->name, QCheckListItem::CheckBox)
{
    setText(1, d->author);
    setText(2, d->comment);
    pd = d;
    setOn(pd->enabled);
}

VideoPluginListItem::~VideoPluginListItem()
{
}



VideoWidgetImpl::VideoWidgetImpl(QWidget* parent, QtVision* qtv)
                :VideoWidget(parent), _qtv(qtv)
{
}

VideoWidgetImpl::~VideoWidgetImpl()
{
}

void VideoWidgetImpl::setup(ConfigData* cfg, KConfig*)
{
    _videoListView->clear();
    for (PluginDesc *p = _qtv->pluginFactory()->videoPlugins().first();
	 p;
	 p = _qtv->pluginFactory()->videoPlugins().next())
    {
	_videoListView->insertItem(new VideoPluginListItem(_videoListView, p));
    }
    selectionChanged();

    connect(_configure, SIGNAL(clicked()),
	    this, SLOT(configurePlugin()));
    connect(_videoListView, SIGNAL(doubleClicked(QListViewItem*)),
	    this, SLOT(itemDoubleClicked(QListViewItem*)));
    connect(_videoListView, SIGNAL(selectionChanged()),
	    this, SLOT(selectionChanged()));

    _fixAR->setChecked(cfg->fixAR);
    _aspectCombo->setCurrentItem(cfg->ARmode);
    _mirror->setChecked(cfg->mirror);
    _landscape->setChecked(cfg->landscape);
    _frameRate->setValue(cfg->frameRate);
}

void VideoWidgetImpl::selectionChanged()
{
    QListViewItem* item = _videoListView->currentItem();
    PluginDesc* p = static_cast<VideoPluginListItem*>(item)->pd;
    _configure->setEnabled(p->configurable);
}

void VideoWidgetImpl::itemDoubleClicked(QListViewItem*)
{
    configurePlugin();
}

void VideoWidgetImpl::configurePlugin()
{
    KDialog*        kdb      = new KDialog(this, "Video source plugin config panel");
    QGridLayout*    grid     = new QGridLayout(kdb, 10, 10);
    KPushButton*    ok_b     = new KPushButton(i18n("&Ok"), kdb);
    KPushButton*    cancel_b = new KPushButton(i18n("&Cancel"), kdb);
    QSplitter*      sp       = new QSplitter(QSplitter::Horizontal, kdb);

    PluginDesc*     pd       = static_cast<VideoPluginListItem*>(_videoListView->currentItem())->pd;
    if (!pd->configurable) return;

    bool previousState       = pd->enabled;
    pd->enabled              = true;
    QVSourcePlugin* p        = _qtv->pluginFactory()->getSourcePlugin(pd, _qtv->screen());
    pd->enabled              = previousState;
    p->probeDevices();

    QWidget*        mw       = p->configWidget(kdb, "Video source configuration widget");

    grid->addMultiCellWidget(sp, 8, 8, 0, 9);
    grid->addMultiCellWidget(ok_b, 9, 9, 7, 7);
    grid->addMultiCellWidget(cancel_b, 9, 9, 9, 9);
    grid->addMultiCellWidget(mw, 0, 7, 0, 9);
    kdb->setCaption(i18n("Configuration"));
    QSize ms(300,200);
    QSize nh = kdb->sizeHint().expandedTo(ms);
    grid->setResizeMode(QLayout::FreeResize);
    kdb->setMinimumSize(nh);
    kdb->resize(nh);

    connect(ok_b, SIGNAL(clicked()),
	    kdb, SLOT(accept()));
    connect(cancel_b, SIGNAL(clicked()),
	    kdb, SLOT(reject()));
    
    if (kdb->exec() == QDialog::Accepted) {
	p->saveConfig();
    }
    p->destroy();
}

void VideoWidgetImpl::apply(ConfigData* cfg, KConfig* h)
{
    cfg->mirror    = _mirror->isChecked();
    cfg->fixAR     = _fixAR->isChecked();
    cfg->ARmode    = _aspectCombo->currentItem();
    cfg->frameRate = _frameRate->value();
    cfg->landscape = _landscape->isChecked();

    h->setGroup("Video Plugins");
    for (QListViewItemIterator it(_videoListView);
	 it.current();
	 it++) {
	VideoPluginListItem* item = static_cast<VideoPluginListItem*>(it.current());
        item->pd->enabled = item->isOn();
	h->writeEntry(item->pd->name + "-" + item->pd->author, item->pd->enabled);
    }

    // make qtvision rescan plugins
}


#include "videowidgetimpl.moc"
